#include <avr/io.h>

#include "Arduino.h"
#include "config.h"
#include "def.h"
#include "types.h"
#include "MultiWii.h"
#include "Alarms.h"
#include "EEPROM.h"
#include "IMU.h"
#include "LCD.h"
#include "Output.h"
#include "RX.h"
#include "Sensors.h"
#include "Serial.h"
#include "GPS.h"
#include "Protocol.h"

#include "EQ.h"
#include "TMC2208.h"

#include "Config.Classic.h"
#include "FPoint.h"
#include "Globals.h"
#include "Command.h"

#include <avr/pgmspace.h>

uint32_t ONE_STEP = 38475;  // 86164 / (48 * 16 * 2916) * 1000000 = 38475
// uint32_t ONE_STEP = 200;  // Test

uint32_t nOldMicro = 0, nNewMicro = 0;

void setup()
{
  EQInit();
  ButtonInit();  

  // Open serial port.
  SerialOpen(0, SERIAL0_COM_SPEED);

  nOldMicro = micros();
  nNewMicro = nOldMicro;
}

void loop()
{
  serialComTelescope();
  processCommands();    // LX200

  nNewMicro = micros();

  if ((nNewMicro - nOldMicro) > (ONE_STEP * 1))
  {
    unsigned long nLonStep = (nNewMicro - nOldMicro) / ONE_STEP;
    nOldMicro += nLonStep * ONE_STEP;

    AutoTrace(nLonStep);
  }
  
  ButtonLoop();
  PhotoLoop();
  EQLoop();
}


#define BTN_WEST    A0
#define BTN_EAST    A1
#define BTN_STOP    A2

uint32_t  nNewButtonMicro = 0, nOldButtonMicro = 0;

uint32_t  STEP_DELAY = 36;
uint32_t  BUTTON_ONE_STEP = 75000;
int nLock = 0;

void ButtonInit()
{
  // 按键模块没有任何上下拉，按键按下时与 GND 短接，所以需要设置内部上拉。
  pinMode(BTN_WEST, INPUT_PULLUP);
  pinMode(BTN_EAST, INPUT_PULLUP);
  pinMode(BTN_STOP, INPUT_PULLUP);

  nOldButtonMicro = micros();
  nNewButtonMicro = nOldMicro;
}

void ButtonLoop()
{
  int nWest = digitalRead(BTN_WEST);
  int nEast = digitalRead(BTN_EAST);
  int nStop = digitalRead(BTN_STOP);
  double nF0 = 0, nF1 = 0;

  nNewButtonMicro = micros();
  if (nNewButtonMicro - nOldButtonMicro > BUTTON_ONE_STEP)
  {
    nOldButtonMicro = micros();
    if (nLock <= 0)
    {
      if (nEast == 0)
      {
        nLock = 10;
        getEqu(&nF0, &nF1, false);
        goToEqu(nF0 + TRIM, nF1);
      }

      if (nWest == 0)
      {
        nLock = 10;
        getEqu(&nF0, &nF1, false);
        goToEqu(nF0 - TRIM, nF1);
      }

      if (nStop == 0)
      {
        nLock = 10;
        EQStop();
      }
    }
    else
    {
      nLock--;
    }
  }
}
